


--[[
--add a menu item to the tools menu
renoise.tool():add_menu_entry {
  name = "Main Menu:Tools:Automation Cleaner",
  invoke = function() CleanAutomation() end 
}
]]

--add a menu item to the context menu in the parameter list  in the Automation view 
renoise.tool():add_menu_entry {
  name = "Track Automation List:Automation Cleaner",
  invoke = function() CleanAutomation() end 
}


--add a menu item to the context menu in the Automation editor 
renoise.tool():add_menu_entry {
  name = "Track Automation:Automation Cleaner",
  invoke = function() CleanAutomation() end 
}

----------------------------------------
function CleanAutomation()
  
  local selected_parameter = renoise.song().selected_parameter
  local selected_pattern_track = renoise.song().selected_pattern_track
  
  --if the automation editor is not active, quit
  if not (renoise.app().window.active_lower_frame == 2) then
    renoise.app():show_status("Automation Cleaner only works when the automation editor is visible")
    return
  end
  
  --if a parameter is selected, get its automation data
  if (selected_parameter) then
    local selected_parameters_automation = selected_pattern_track:find_automation(selected_parameter)
    
    --if there is no automation data, quit  
    if (not selected_parameters_automation) then
      renoise.app():show_status("No automation points for current parameter in current pattern")
      return
    end
    
    local pts = selected_parameters_automation.points
  
    --These two arrays are necessary because the points have to be sorted by time
    local times = {} --contains only times, will later be sorted by time
    local temparray = {} --will be used to look up the value for each time point
    
    for a,b in pairs(pts) do
      table.insert(temparray, b.time, b.value)
      table.insert(times, b.time)
    end
    
    --sort the array of times
    table.sort(times)
    
    --do this for all points, except the first and the last
    for i = 2, ((# times)-1)do
      local tim = times[i]
      local prevtim = times[i-1]
      local nexttim = times[i+1]
    
      --if the value of a point lies between the values of the preceding and following points, delete it.
      if (temparray[tim] >= temparray[prevtim]) and (temparray[tim] <= temparray[nexttim])then
        selected_parameters_automation:remove_point_at(tim)
      elseif (temparray[tim] <= temparray[prevtim]) and (temparray[tim] >= temparray[nexttim])then
        selected_parameters_automation:remove_point_at(tim)
      end
    end
    
    --make it smooth
    selected_parameters_automation.playmode = renoise.PatternTrackAutomation.PLAYMODE_CUBIC
  
  --if no parameter is selected, quit
  else
    renoise.app():show_status("No parameter selected in the automation editor")
    return
  end
end  
